@@ -42,16 +42,18 @@ CREATE TABLE Member (
 -- Messages and Threads
 CREATE TABLE Message (
 	id INT PRIMARY KEY REFERENCES Resource,
-	msg_id TEXT,   -- for email/news threading only
-	parent INT REFERENCES Message, -- In-Reply-To:
-	thread INT NOT NULL,
-	title TEXT NOT NULL,   -- Subject:
+	parent INT REFERENCES Message,   -- In-Reply-To:
+	version_of INT REFERENCES Message,   -- current version
+	open BOOLEAN DEFAULT false,   -- editing open for all members
 	creator INT NOT NULL REFERENCES Member,   -- From:
+	title TEXT NOT NULL,   -- Subject:
 	format TEXT,   -- MIME type
 	content TEXT);
 
 CREATE INDEX Message_parent_idx ON Message (parent);
 
+CREATE INDEX Message_version_of_idx ON Message (version_of);
+
 -- Voting Data
 CREATE TABLE Vote (
 	id INT PRIMARY KEY REFERENCES Resource,
