@@ -11,32 +11,13 @@
 
 require 'samizdat'
 
-session = Session.new
-
-session.response do |t|
-    set_lang, = session.params %w[set_lang]
-    if set_lang then
-        session.set_cookie('lang', set_lang)
-        session.redirect
-    end
-
+request do |session|
+    t = session.template
     skip, skip_focus = session.params %w[skip skip_focus]
     skip = skip.to_i
     skip_focus = skip_focus.to_i
 
-    messages = session.rdf.select_all( %{
-SELECT ?msg
-WHERE (dc::date ?msg ?date)
-      (s::thread ?msg ?msg)
-ORDER BY ?date DESC
-USING PRESET NS}, limit_page, limit_page * skip
-    ).collect {|msg,| Resource.new(session, msg).render }
-
-    page = [ [ _('Recent Updates') +
-            (skip > 0? sprintf(_(', page %s'), skip + 1) : ''),
-        t.list(messages, t.nav(skip + 1)) ] ]
-
-    table = session.rdf.select_all( %{
+    table = rdf.select_all( %{
 SELECT ?focus, count(?resource)
 WHERE (rdf::predicate ?stmt dc::relation)
       (rdf::subject ?stmt ?resource)
@@ -48,24 +29,24 @@ ORDER BY count(?resource) DESC
 USING PRESET NS}, limit_page, limit_page * skip_focus
     ).collect {|focus, usage| [ Resource.new(session, focus).render, usage ] }
 
-    page << [ _('Important Focuses') +
+    page = [ [ _('Main Focuses') +
             (skip_focus > 0? sprintf(_(', page %s'), skip_focus + 1) : ''),
         t.table([[_('Focus'), _('Related Resources')]] + table,
-            t.nav(skip_focus + 1, 'index.rb?', 'skip_focus')) ]
+            t.nav(skip_focus + 1, 'index.rb?', 'skip_focus')) ] ]
 
-    side = ''
-    if defined?(GetText) then   # add language selection box
-        old_language = session.language
-        list = ''
-        config['locale']['languages'].each do |lang|
-            lang.untaint
-            session.language = lang
-            list += %{<a href="index.rb?set_lang=#{lang}">} +
-                _('English') + "</a>\n"
-        end
-        session.language = old_language
-        side = t.box(_('Interface Language'), list)
-    end
+    messages = rdf.select_all( %{
+SELECT ?msg
+WHERE (dc::date ?msg ?date)
+      (s::inReplyTo ?msg ?parent)
+      (dct::isVersionOf ?msg ?current)
+LITERAL ?parent IS NULL AND ?current IS NULL
+ORDER BY ?date DESC
+USING PRESET NS}, limit_page, limit_page * skip
+    ).collect {|msg,| Resource.new(session, msg).render }
 
-    t.page(_('Front Page'), page, side)
+    page << [ _('Recent Updates') +
+            (skip > 0? sprintf(_(', page %s'), skip + 1) : ''),
+        t.list(messages, t.nav(skip + 1)) ]
+
+    t.page(_('Front Page'), page)   # todo: language_box for new users
 end
