@@ -11,7 +11,7 @@
 
 require 'samizdat'
 
-session = Session.new
-
-session.close if session.id
-session.redirect
+request do |session|
+    session.close if session.id
+    session.redirect
+end
