@@ -11,9 +11,12 @@
 
 require 'samizdat'
 
-session = Session.new
-
-session.response do |t|
+request do |session|
+    set_lang, = session.params %w[set_lang]
+    if set_lang then
+        session.set_cookie('lang', set_lang)
+        session.redirect
+    end
 
     # account settings
     login, full_name, email, passwd1, passwd2 = \
@@ -97,5 +100,8 @@ end # transaction
         [:label], [:submit, 'submit']
     )
 
-    t.page(title, t.form('member.rb', *form))
+    t = session.template
+    t.page(_('Member Settings'),
+        t.box(title, t.form('member.rb', *form)) +
+        t.language_box)
 end
