@@ -27,11 +27,13 @@ server.add_handler('pingback.ping') do |source, target|
 
     # check if target exists and is properly pingback-enabled
     base = 'http://' + ENV['HTTP_HOST'] + config['site']['base'] + '/'
-    rdf = SamizdatRDF.new(base)
-    title, thread = rdf.select_one %{
-SELECT ?title, ?thread
-WHERE (dc::title #{target} ?title)
-      (s::thread #{target} ?thread)
+    target_id = target.sub(Regexp.new('^'+base), '').to_i
+    target_id = 0 unless base + target_id.to_s == target
+    title, = rdf.select_one %{
+SELECT ?title
+WHERE (dc::title #{target_id} ?title)
+      (dct::isVersionOf #{target_id} ?current)
+LITERAL ?current IS NULL
 USING PRESET NS}
     raise FaultException.new(0x0021, "Target doesn't support pingback") if
         title.nil?
@@ -51,9 +53,8 @@ USING PRESET NS}
     ) unless response.body.include? target
 
     # check if pingback has not already been registered
-    pattern = %{(s::inReplyTo ?pingback #{target})
-      (s::thread ?pingback #{base}#{thread})
-      (dc::creator ?pingback #{base}#{member})
+    pattern = %{(s::inReplyTo ?pingback #{target_id})
+      (dc::creator ?pingback #{member})
       (dc::format ?pingback 'text/uri-list')
       (dc::title ?pingback ?title)
       (s::content ?pingback ?content)
