@@ -11,11 +11,10 @@
 
 require 'samizdat'
 
-session = Session.new
-
-session.response do |t|
+request do |session|
     namespaces = ns   # standard namespaces
 
+    t = session.template
     query, substring, skip = session.params %w[query substring skip]
     skip = skip.to_i
 
@@ -78,7 +77,7 @@ USING dc FOR #{ns['dc']}}
     # phase 2: validate query
 
     begin
-        q = SquishQuery.new(session.rdf, query)
+        q = SquishQuery.new(query)
     rescue ProgrammingError
         raise UserError, _('Error in your query: ') + $!
     end
@@ -95,11 +94,11 @@ _('Must-bind list should contain only one blank node, filters based on queries w
         title = _('Search Result') +
             (skip > 0? sprintf(_(', page %s'), skip + 1) : '')
         begin
-            sql = session.rdf.select(q)
+            sql = rdf.select(q)
         rescue ProgrammingError
             raise UserError, _('Error in your query: ') + $!
         end
-        page = session.rdf.select_all(
+        page = rdf.select_all(
             q, limit_page, limit_page * skip
         ).collect {|id,| Resource.new(session, id).render }
         page =
