@@ -7,7 +7,7 @@
 #   the GNU General Public License version 2 or later.
 #
 
-SAMIZDAT_VERSION = '0.5.1'
+SAMIZDAT_VERSION = '0.5.2'
 
 # :main:samizdat.rb
 
@@ -71,7 +71,7 @@ class SamizdatConfig
 
     def initialize
         require 'yaml'
-        @config = YAML::load(File::open('config.yaml'))
+        @config = File.open('config.yaml') {|f| YAML.load(f) }
     end
 
     attr_reader :config
