@@ -21,15 +21,15 @@ class Focus < Resource
         end
         if id.to_i == 0 then   # derive id from uriref
             uriref = id
-            id, = resource.rdf.select_one(
-                %{SELECT ?focus WHERE (s::id #{id} ?focus) USING PRESET NS})
+            id, = rdf.select_one(
+                %{SELECT ?focus WHERE (s::id #{uriref} ?focus) USING PRESET NS})
         end
-        super resource.session, id
+        super resource.session, id   # todo: unchild from Resource
         @id = @uriref = uriref if id.nil? and uriref   # keep external uriref
 
         if @id.kind_of? Integer then
             begin
-                @name = render(:short)[1]
+                @name = render(:short)[1]   # watch out for recursion!
             rescue ResourceNotFoundError   # bad id in focus cookie
                 session.set_cookie('focus', nil, 0)   # delete bad cookie
                 session.redirect(session.base)
@@ -57,9 +57,9 @@ class Focus < Resource
         if @rating.nil? then
             @rating, = rdf.select_one %{
 SELECT ?rating
-WHERE (rdf::subject ?stmt #{resource.uriref})
+WHERE (rdf::subject ?stmt #{resource.id})
       (rdf::predicate ?stmt dc::relation)
-      (rdf::object ?stmt #{uriref})
+      (rdf::object ?stmt #{@id})
       (s::rating ?stmt ?rating)
 USING PRESET NS}
             @rating = @rating ? @rating.to_f : _('none')
@@ -72,13 +72,13 @@ USING PRESET NS}
     def rating=(value)
         value = Focus.validate_rating(value)
         if value then
-            resource.rdf.assert %{
+            rdf.assert %{
 UPDATE ?rating = '#{value}'
-WHERE (rdf::subject ?stmt #{resource.uriref})
+WHERE (rdf::subject ?stmt #{resource.id})
       (rdf::predicate ?stmt dc::relation)
-      (rdf::object ?stmt #{uriref})
+      (rdf::object ?stmt #{@id})
       (s::voteProposition ?vote ?stmt)
-      (s::voteMember ?vote #{session.base}#{session.id})
+      (s::voteMember ?vote #{session.id})
       (s::voteRating ?vote ?rating)
 USING PRESET NS}
             @rating = nil   # invalidate rating cache
