@@ -11,7 +11,6 @@ class Resource
     def initialize(session, id=nil)
         @session = session
         @template = session.template
-        @rdf = session.rdf
         case id
         when Integer then @id = id
         else
@@ -20,13 +19,14 @@ class Resource
         @uriref = session.base + id.to_s
     end
 
-    attr_reader :session, :template, :rdf, :id, :uriref
+    attr_reader :session, :template, :id, :uriref
 
     # in :list mode (default) return rendered desc
     #
     # in :short mode return type, head, and unrendered desc
     #
-    # in :full mode, add related resources to desc
+    # in :full mode, add related focuses and resources to desc
+    # (only one full resource per page is possible)
     #
     def render(mode=:list)
         if self.class == Focus and id.to_i == 0 then
@@ -41,6 +41,7 @@ class Resource
             head.gsub!(/\Afocus::/, '') and head = _(head)   # focus is special
             desc = _('refers to <a href="%s">external uriref</a>') % label
             if mode == :full then
+                desc = '<p>' + desc + '</p>' + render_focuses(head, mode)
                 # todo: select all statements with this subject
             end
         elsif literal then
@@ -50,15 +51,16 @@ class Resource
 %{<a href="#{uriref}">#{SquishQuery.ns_shrink(uriref)}</a> = #{label}}
         else   # internal resource
             type = label
-            head, desc = case label
-            when nil then raise ResourceNotFoundError, id.to_s
-            when 'Member' then render_member mode
-            when 'Message' then render_message mode
-            when 'Statement' then render_statement mode
-            when 'Vote' then render_vote mode
-            when 'Item' then render_item mode
-            else raise RuntimeError, _("Unknown resource type '%s'") % label
-            end
+            head, desc =
+                case label
+                when nil then raise ResourceNotFoundError, id.to_s
+                when 'Member' then render_member mode
+                when 'Message' then render_message mode
+                when 'Statement' then render_statement mode
+                when 'Vote' then render_vote mode
+                when 'Item' then render_item mode
+                else raise RuntimeError, _("Unknown resource type '%s'") % label
+                end
         end
         case mode
         when :list then return template.resource(id, head, desc)
@@ -77,8 +79,8 @@ class Resource
             if id.kind_of? Integer and id > 0 then
                 format, login = rdf.select_one %{
 SELECT ?format, ?login
-WHERE (dc::format #{uriref} ?format)
-      (dc::creator #{uriref} ?creator)
+WHERE (dc::format #{id} ?format)
+      (dc::creator #{id} ?creator)
       (s::login ?creator ?login)
 USING PRESET NS}
             else
@@ -91,13 +93,38 @@ USING PRESET NS}
         config['site']['content'] + '/' + login + '/' + id.to_s + '.' + ext
     end
 
+    # multimedia message content filename (wrapper around content_location)
+    #
+    def content_filename(id, format=nil, login=nil)
+        session.filename(content_location(id, format, login))
+    end
+
 private
 
+    def render_focuses(head, mode)
+        current = Focus.new(self)
+        focuses = (:full == mode)? {current.id => current} : {}
+        rdf.select_all( %{
+SELECT ?focus
+WHERE (rdf::subject ?stmt #{id})
+      (rdf::predicate ?stmt dc::relation)
+      (rdf::object ?stmt ?focus)
+      (s::rating ?stmt ?rating)
+LITERAL ?rating > 0
+ORDER BY ?rating DESC
+USING PRESET NS}, limit_page).collect {|f,| f }.each {|focus|
+            f = Focus.new(self, focus)
+            focuses[f.id] = f
+        }
+        (:full == mode)? template.focus_box(id, head, focuses.values) :
+            template.focus_line(id, focuses.values)
+    end
+
     def render_member(mode)
         login, head = rdf.select_one "
 SELECT ?login, ?full_name
-WHERE (s::login #{uriref} ?login)
-      (s::fullName #{uriref} ?full_name)
+WHERE (s::login #{id} ?login)
+      (s::fullName #{id} ?full_name)
 USING s FOR #{ns['s']}"
         desc = _('Login') +": #{login}"
         if mode == :full then
@@ -106,11 +133,14 @@ USING s FOR #{ns['s']}"
             messages = rdf.select_all( %{
 SELECT ?msg
 WHERE (dc::date ?msg ?date)
-      (dc::creator ?msg #{uriref})
+      (dc::creator ?msg #{id})
+      (dct::isVersionOf ?msg ?current)
+LITERAL ?current IS NULL
 ORDER BY ?date DESC
 USING PRESET NS}, limit_page, limit_page * skip
             ).collect {|msg,| Resource.new(session, msg).render }
             desc = template.box(nil, desc)
+            desc << render_focuses(head, mode)
             desc << template.box( _('Latest Messages') +
                 (skip > 0? sprintf(_(', page %s'), skip + 1) : ''),
                 template.list(messages,
@@ -123,45 +153,68 @@ USING PRESET NS}, limit_page, limit_page * skip
     def render_message(mode)
         date, head, creator, name, login = rdf.select_one %{
 SELECT ?date, ?title, ?creator, ?name, ?login
-WHERE (dc::date #{uriref} ?date)
-      (dc::title #{uriref} ?title)
-      (dc::creator #{uriref} ?creator)
+WHERE (dc::date #{id} ?date)
+      (dc::title #{id} ?title)
+      (dc::creator #{id} ?creator)
       (s::fullName ?creator ?name)
       (s::login ?creator ?login)
 USING PRESET NS}
-        focus = Focus.new(self) unless self.class == Focus
-        return head, template.message_info( :id => id, :date => date,
-                :creator => creator, :full_name => name, :focus => focus
-            ) if :list == mode
+        replies, = rdf.select_one "
+SELECT count(?msg) WHERE (s::inReplyTo ?msg #{id}) USING PRESET NS"
+        focus = render_focuses(head, mode) unless :full == mode or
+            Focus == self.class   # avoid recursion
+        return head, template.message_info(:id => id, :date => date,
+            :creator => creator, :full_name => name, :focus => focus,
+            :replies => replies) if :list == mode
         # extra data for :short and :full modes
         format, = rdf.select_one "
-SELECT ?format WHERE (dc::format #{uriref} ?format) USING PRESET NS"
+SELECT ?format WHERE (dc::format #{id} ?format) USING PRESET NS"
         content, = rdf.select_one "
-SELECT ?content WHERE (s::content #{uriref} ?content) USING PRESET NS"
+SELECT ?content WHERE (s::content #{id} ?content) USING PRESET NS"
         content = content_location(id, format, login) if content.nil? and
             format and not config['format']['inline'].include? format
-        if mode == :full then   # shouldn't refer to parent in :short mode
-            parent, = rdf.select_one "
-SELECT ?parent WHERE (s::inReplyTo #{uriref} ?parent) USING PRESET NS"
+        if mode == :full then   # show parent and versions only in full mode
+            current, = rdf.select_one "
+SELECT ?current WHERE (dct::isVersionOf #{id} ?current) USING PRESET NS"
+            if current.nil? then   # non-current versions don't have history
+                parent, = rdf.select_one "
+SELECT ?parent WHERE (s::inReplyTo #{id} ?parent) USING PRESET NS"
+                history, = rdf.select_one "
+SELECT count(?version) WHERE (dct::isVersionOf ?version #{id}) USING PRESET NS"
+                if creator != session.id then
+                    open, = rdf.select_one "
+SELECT ?open WHERE (s::openForAll #{id} ?open) USING PRESET NS"
+                else 
+                    open = true
+                end
+            end
         end
         desc = template.message(:id => id, :date => date, :creator => creator,
-            :full_name => name, :focus => focus, :title =>  head, :format =>
-            format, :content => content, :parent => parent, :mode => mode)
+            :full_name => name, :focus => focus, :title => head,
+            :format => format, :content => content, :parent => parent,
+            :replies => replies, :current => current, :history => history,
+            :open => open, :mode => mode)
         if mode == :full   # add replies
+            desc << render_focuses(head, mode)
             skip, = session.params %w[skip]
             skip = skip.to_i
             replies = rdf.select_all( %{
 SELECT ?msg
 WHERE (dc::date ?msg ?date)
-      (s::inReplyTo ?msg #{uriref})
+      (s::inReplyTo ?msg #{id})
 ORDER BY ?date
 USING PRESET NS}, limit_page, limit_page * skip
             ).collect {|msg,| Resource.new(session, msg).render(:short)[2] }
-            desc << template.box( _('Replies') +
-                (skip > 0? sprintf(_(', page %s'), skip + 1) : ''),
+            desc << template.box(
+                _('Replies') +
+                    (skip > 0? sprintf(_(', page %s'), skip + 1) : ''),
                 template.list(replies,
-                    template.nav(skip + 1, %{resource.rb?id=#{id}&amp;}))
+                    template.nav(skip + 1, %{resource.rb?id=#{id}&amp;})),
+                'replies'
             ) if replies.size > 0
+            # navigation links
+            template.link['made'] = creator
+            template.link['up'] = parent if parent
         end
         return head, desc
     end
@@ -170,9 +223,9 @@ USING PRESET NS}, limit_page, limit_page * skip
         head = _('Statement') + ' ' + id.to_s
         stmt = rdf.select_one %{
 SELECT ?p, ?s, ?o
-WHERE (rdf::predicate #{uriref} ?p)
-      (rdf::subject #{uriref} ?s)
-      (rdf::object #{uriref} ?o)
+WHERE (rdf::predicate #{id} ?p)
+      (rdf::subject #{id} ?s)
+      (rdf::object #{id} ?o)
 USING rdf FOR #{ns['rdf']}}
         n = [_('Predicate'), _('Subject'), _('Object')]
         if mode == :list then
@@ -185,6 +238,7 @@ USING rdf FOR #{ns['rdf']}}
             }.join
         end
         if mode == :full then
+            desc << render_focuses(head, mode)
             # todo: display votes
         end
         return head, desc
@@ -194,15 +248,17 @@ USING rdf FOR #{ns['rdf']}}
         head = _('Vote') + ' ' + id.to_s
         stmt, member, name, rating = rdf.select_one "
 SELECT ?stmt, ?member, ?name, ?rating
-WHERE (s::voteProposition #{uriref} ?stmt)
-      (s::voteMember #{uriref} ?member)
+WHERE (s::voteProposition #{id} ?stmt)
+      (s::voteMember #{id} ?member)
       (s::fullName ?member ?name)
-      (s::voteRating #{uriref} ?rating)
+      (s::voteRating #{id} ?rating)
 USING s FOR #{ns['s']}"
         desc = sprintf(_('<a href="%s">%s</a> gives rating %4.2f to the <a href="%s">Statement %s</a>.'), member, name, rating, stmt, stmt)
         if mode == :full then
             desc = template.box(nil, desc) + template.box(_('Vote Proposition'),
-                Resource.new(session, stmt).render(:full)[2])
+                Resource.new(session, stmt).render(:short)[2]) +
+                render_focuses(head, mode)
+            template.link['made'] = member
         end
         return head, desc
     end
@@ -213,7 +269,7 @@ USING s FOR #{ns['s']}"
             contributor, c_name, possessor, p_name = rdf.select_one "
 SELECT ?msg, ?date, ?creator, ?name, ?title, ?content, ?parent,
        ?contributor, ?c_name, ?possessor, ?p_name
-WHERE (s::description #{uriref} ?msg)
+WHERE (s::description #{id} ?msg)
       (dc::date ?msg ?date)
       (dc::creator ?msg ?creator)
       (s::fullName ?creator ?name)
@@ -221,9 +277,9 @@ WHERE (s::description #{uriref} ?msg)
       (dc::format ?msg ?format)
       (s::content ?msg ?content)
       (s::inReplyTo ?msg ?parent)
-      (s::contributor #{uriref} ?contributor)
+      (s::contributor #{id} ?contributor)
       (s::fullName ?contributor ?c_name)
-      (s::possessor #{uriref} ?possessor)
+      (s::possessor #{id} ?possessor)
       (s::fullName ?possessor ?p_name)
 USING dc FOR http://purl.org/dc/elements/1.1/
       s FOR #{ns['s']}"
@@ -237,7 +293,7 @@ USING dc FOR http://purl.org/dc/elements/1.1/
         if mode == :full then
             query = "
 SELECT ?item, ?name
-WHERE (s::description #{uriref} ?msg)
+WHERE (s::description #{id} ?msg)
       (s::description ?item ?msg)
       (s::possessor ?item ?possessor)
       (s::fullName ?possessor ?name)
