@@ -10,6 +10,22 @@
 require 'delegate'
 require 'digest/md5'
 
+begin
+    raise LoadError if defined?(MOD_RUBY)
+    require 'fcgi'
+    def request
+        FCGI.each_cgi do |cgi|
+            catch :finish do
+                Session.new(cgi).response {|s| yield s }
+            end
+        end
+    end
+rescue LoadError
+    def request
+        Session.new(CGI.new).response {|s| yield s }
+    end
+end
+
 # session management and CGI parameter handling
 #
 class Session < SimpleDelegator
@@ -37,13 +53,13 @@ class Session < SimpleDelegator
         })
     end
 
-    # translate location to a real file name (uses Apache/mod_ruby specifics)
+    # translate location to a real file name
     #
     def filename(location)
         if defined?(MOD_RUBY) then
             Apache.request.lookup_uri(location).filename
         else
-            ENV['DOCUMENT_ROOT'] + location
+            @cgi.env_table['DOCUMENT_ROOT'] + location
         end
     end
 
@@ -65,20 +81,22 @@ class Session < SimpleDelegator
     #
     # set id and refresh session if session cookie is valid
     #
-    def initialize
-        @cgi = CGI.new
+    def initialize(cgi)
+        @cgi = cgi
+        class << @cgi
+            public :env_table
+        end
         @options = {'charset' => 'utf-8', 'cookie' => []}
 
         self.language = cookie('lang')
 
-        proto = ENV['HTTPS'] ? 'https' : 'http'
-        port = ENV['SERVER_PORT'].to_i
+        proto = @cgi.env_table['HTTPS'] ? 'https' : 'http'
+        port = @cgi.env_table['SERVER_PORT'].to_i
         port = (port == {'http' => 80, 'https' => 443}[proto]) ?
             '' : ':' + port.to_s
         @base = proto + '://' + @cgi.host + port + config['site']['base'] + '/'
 
         @template = Template.new(self)
-        @rdf = SamizdatRDF.new(@base)
 
         # check session
         @session = cookie('session')
@@ -111,9 +129,6 @@ end
     # Template object aware of this session's options
     attr_reader :template
 
-    # SamizdatRDF object aware of this session's options
-    attr_reader :rdf
-
     attr_reader :id, :login, :full_name, :email
 
     # open new session on login
@@ -130,7 +145,7 @@ end
                 session = ? WHERE id = ?", @session, @id
                 db.commit
                 set_cookie('session', @session, @@last_timeout)
-                redirect
+                redirect(base)
             else
                 response() { template.page(_('Login Failed'),
                     '<p>'+_('Wrong login name or password. Try again.')+'</p>')
@@ -187,11 +202,10 @@ end
         if block_given? then
             page =
                 begin
-                    yield template
+                    yield self
                 rescue AuthError
                     @options['status'] = 'AUTH_REQUIRED'
-                    template.page( _('Please Login'),
-%{<p>#{$!}.</p>\n<p>}+_("Use the form in the sidebar to login, if you are a registered member, or to create a new account, if you don't have an account on this site yet.")+'</p>')
+                    template.page(_('Please Login'), %{<p>#{$!}.</p>})
                 rescue UserError
                     template.page(_('User Error'),
 %{<p>#{$!}.</p><p>}+_("Press 'Back' button of your browser to return.")+'</p>')
@@ -206,18 +220,19 @@ end
 <pre>#{caller.join("\n")}</pre>
 <p>}+_('Please report this error to the site administrator.')+'</p>')
                 end
-            @cgi.out(@options) { page }
+            print @cgi.out(@options) { page }
         else
             print @cgi.header(@options)
         end
-        @rdf = nil
-        @template = nil
-        @cgi = nil
-        GC.start
-        exit
+        if defined?(FCGI) then
+            throw :finish
+        else
+            exit
+        end
     end
 
     def redirect(location=referer)
+        location = base if location.nil?
         response({'status' => 'REDIRECT', 'location' => location})
     end
 
