@@ -48,22 +48,21 @@ end
 # provides access to RDF storage via DBI-like interface
 #
 class SamizdatRDF
+    include Singleton
+
     # initialize class attributes
     #
-    def initialize(base)
-        @internal = Regexp.new(/\A#{base}(\d+)\z/).freeze
-
+    def initialize
         @map = {}
         config['map'].each_pair {|p, m|
             @map[ p.sub(/\A(\S+?)::/) { config['ns'][$1] } ] = m
         }
 
+        @select_cache = {}
+
         @db = Database.instance.db
     end
 
-    # regexp to detect internal resource id from URI starting with site base
-    attr_reader :internal
-
     # cached internal property map with expanded namespaces
     attr_reader :map
 
@@ -86,16 +85,14 @@ class SamizdatRDF
         end
     end
 
-    @@select_cache = {}
-
     # accepts String or pre-parsed SquishQuery object, caches SQL by String
     #
     def select(query)
         case query
         when String
-            return @@select_cache[query] if @@select_cache.has_key? query
-            @@select_cache = {} if @@select_cache.size >= 1024
-            @@select_cache[query] = SquishQuery.new(self, query).to_sql
+            return @select_cache[query] if @select_cache.has_key? query
+            @select_cache = {} if @select_cache.size >= 1024
+            @select_cache[query] = SquishQuery.new(query).to_sql
         when SquishQuery
             query.to_sql
         else
@@ -106,7 +103,7 @@ class SamizdatRDF
     # merge Squish query into RDF database
     #
     def assert(query)
-        q = SquishQuery.new(self, query)
+        q = SquishQuery.new(query)
         raise ProgrammingError, 'Wrong query type: assert expected' unless
             q.type == :assert
         insert, update = q.nodes
@@ -117,7 +114,7 @@ class SamizdatRDF
         v = {}   # v: node -> value
         new = {} # new[node] if node was inserted at this stage
         q.pm.each do |node, map|
-            if node =~ @internal then   # internal resource
+            if node =~ SquishQuery::INTERNAL then   # internal resource
                 v[node] = $1
             elsif node =~ SquishQuery::LITERAL then   # string literal
                 v[node] = node
@@ -125,11 +122,12 @@ class SamizdatRDF
                 # blank node only in object position
                 v[node] = update[node]   # todo: what if update[node].nil?
             elsif node =~ SquishQuery::BN then   # resource blank node
-                s = SquishQuery.new(self, subgraph(node, q.pattern))
+                s = SquishQuery.new(subgraph(node, q.pattern))
                 v[node], = db.select_one q.unstring(
-                        "SELECT #{q.b[node]} FROM "+
-                        s.table.collect {|a,t| t+' '+a }.join(', ')+
-                        ((s.jc.length > 0)? ' WHERE '+s.jc.join(' AND ') : ''))
+                    "SELECT #{s.b[node]} FROM " +
+                    s.table.collect {|a,t| t + ' ' + a }.join(', ') +
+                    ((s.jc.length > 0)? ' WHERE ' + s.jc.join(' AND ') : '')
+                ) unless insert.include?(node)
                 if v[node].nil? then
                     db.do "INSERT INTO Resource (label) VALUES (?)",
                         q.c[map.rassoc(0)[0]][0][0]   # subject table
@@ -202,6 +200,10 @@ private
     end
 end
 
+def rdf
+    SamizdatRDF.instance
+end
+
 
 require 'uri/common'
 
@@ -211,6 +213,9 @@ require 'uri/common'
 # functions to deal with Squish syntax
 #
 class SquishQuery
+    # regexp for internal resource reference
+    INTERNAL = Regexp.new(/\A([[:digit:]]+)\z/).freeze
+
     # regexp for blank node mark and name
     BN = Regexp.new(/\A\?([[:alnum:]_]+)\z/).freeze
 
@@ -221,7 +226,7 @@ class SquishQuery
     NUMBER = Regexp.new(/\A-?[[:digit:]]+(\.[[:digit:]]+)?\z/).freeze
 
     # regexp for operator
-    OPERATOR = Regexp.new(/\A(\+|-|\*|\/|<|<=|>|>=|=|I?LIKE|AND|OR)\z/i).freeze
+    OPERATOR = Regexp.new(/\A(\+|-|\*|\/|<|<=|>|>=|=|I?LIKE|AND|OR|IS|NULL)\z/i).freeze
 
     # regexp for aggregate function
     AGGREGATE = Regexp.new(/\A(avg|count|max|min|sum)\z/i).freeze
@@ -239,9 +244,7 @@ class SquishQuery
     # generate query pattern graph, call transform_pattern,
     # determine query type and parse nodes section accordingly
     #
-    def initialize(rdf, query)
-        @rdf = rdf
-
+    def initialize(query)
         if query.class == Array then  # pre-parsed query
             @pattern = query
             transform_pattern
@@ -288,6 +291,10 @@ class SquishQuery
             raise ProgrammingError, "Blank node '#{node}' is not bound by the query pattern" unless @b[node]
         end
 
+        @literal.scan(/\?\b.+?\b/) do |node|
+            @jc.delete(@b[node]+' IS NOT NULL')   # treat it as ground
+        end
+
         # determine query type, parse and validate nodes section
         if @key == 'SELECT' then
             @type = :select
@@ -401,16 +408,13 @@ class SquishQuery
             p, s, o = pattern[i]
             # validate the triple
             raise ProgrammingError, "Valid uriref expected in predicate position instead of '#{p}'" unless p =~ URI::URI_REF
-            [s, o].each {|node| raise ProgrammingError, "Valid uriref or blank node name expected instead of '#{node}'" unless node =~ URI::URI_REF or node =~ BN }
-            if map = @rdf.map[p] and
-            (s =~ BN or s =~ @rdf.internal or map['Resource']) then
-                # internal predicate and subject
-                c[i] = [
-                    # list of possible mappings into internal tables
+            [s, o].each {|node| raise ProgrammingError, "Resource or blank node name expected instead of '#{node}'" unless node =~ INTERNAL or node =~ URI::URI_REF or node =~ BN }
+            map = rdf.map[p]
+            if map and (s =~ BN or s =~ INTERNAL or map['Resource']) then
+                # internal predicate and subject is mappable to Resource table
+                c[i] = [   # list of possible mappings into internal tables
                     map.keys.collect {|t| [t, 'id'] },   # subject
                     map.keys.collect {|t| [t, map[t]] }  # object
-                    # optimize: table name is always the same for both
-                    # subject and object, don't have to store it twice
                 ]
                 # reverse mapping of the node occurences
                 if pm[s] then pm[s].push [i, 0]
@@ -481,7 +485,7 @@ class SquishQuery
                     ac.succ!
                 end
             end
-        end
+        end   # optimize: unnecessary aliases are generated
 
         pm.each do |node, map|
             # node binding
@@ -499,11 +503,10 @@ class SquishQuery
 
             # ground non-blank nodes
             if node !~ BN then
-                if node =~ @rdf.internal then   # internal resource id
+                if node =~ INTERNAL then   # internal resource id
                     jc.push b[node]+' = '+$1
                 elsif node =~ LITERAL then   # literal
                     jc.push b[node]+' = '+node
-                    # todo: numeric literals
                 # optimize: elsif c[map[0][0]][0] == 'Resource'
                 else   # external resource
                     table[ac] = 'Resource'
