@@ -64,8 +64,8 @@ class TC_Robot < Test::Unit::TestCase
         assert_equal SAMIZDAT_VERSION, version
         # look for login form
         assert_equal 'login.rb',
-            root.elements['/html/body/div[@id="sidebar"]/div/div/form'
-            ].attributes['action']
+            root.elements['/html/body/div[@id="subhead"]/div/a'
+            ].attributes['href']
 
         # test anonymous check on publish
         response = get('message.rb')
@@ -86,14 +86,14 @@ class TC_Robot < Test::Unit::TestCase
             print 'test login already exists'
         when Net::HTTPFound
             assert @@session = response['set-cookie']
-            assert_equal @base.path, response['location']
+            assert_equal @base.to_s, response['location']
 
-            # check if valid session is reflected in sidebar
+            # check if valid session is reflected in subhead
             response = get('', {'Cookie' => @@session})
             assert_equal Net::HTTPOK, response.class
             root = parse(response.body)
-            assert_equal @login, elements(root,
-                '/html/body/div[@id="sidebar"]/div/div[@class="box-title"]')[0]
+            assert_equal @full_name, elements(root,
+                '/html/body/div[@id="subhead"]/div/a[1]')[0]
         else assert false, 'Unexpected HTTP code'
         end
 
@@ -115,7 +115,7 @@ class TC_Robot < Test::Unit::TestCase
         assert_equal Net::HTTPOK, response.class
         root = parse(response.body)
         @@id = root.elements[
-            '/html/body/div[@id="sidebar"]/div/div/p[1]/a'].attributes['href']
+            '/html/body/div[@id="subhead"]/div/a[1]'].attributes['href']
         assert @@id.to_i > 0
     end
 
@@ -175,11 +175,11 @@ class TC_Robot < Test::Unit::TestCase
             {"Cookie" => @@session, "Referer" => @base.path + id})
         assert @@session = response['set-cookie']
         assert_equal Net::HTTPFound, response.class
-        assert_equal @base.path + id, response['location']
+        assert_equal id, response['location']
         assert_equal Net::HTTPOK, (response = get(id)).class
         root = parse(response.body)
-        assert_equal ['1.00'], elements(root,
-            %{/html/body/div[@id="sidebar"]//div[@class="focus"][1]/a[@class="rating"]})
+        assert_equal [': 1.00 ('], elements(root,
+            %{/html/body//div[@id="focuses"]/div[@class="box-content"]/p[1]})
     end
 
     def test_05_stress
@@ -199,7 +199,39 @@ class TC_Robot < Test::Unit::TestCase
             @@session = response['set-cookie']
             count = count - 1
         end
+        # todo: test skip (index.rb, resource.rb)
     end
 
-    # todo: test_06_skip (index.rb, resource.rb)
+    def test_06_pingback
+        # don't test pingback if it's not properly enabled
+        return if config['pingback'].nil? or config['pingback']['login'].nil?
+        id, = db.select_one('SELECT id FROM Member WHERE login = ?',
+            config['pingback']['login'])
+        return if id.nil?
+
+        title, content = 'Test Pingback - Target', '.'
+        response = post('message.rb', "title=#{CGI.escape(title)}&content=#{CGI.escape(content)}&confirm", {"Cookie" => @@session})
+        @@session = response['set-cookie']
+        target = response['location']
+
+        title = 'Test Pingback - Sources'
+        response = post('message.rb', "title=#{CGI.escape(title)}&content=#{CGI.escape(content)}&confirm", {"Cookie" => @@session})
+        @@session = response['set-cookie']
+        parent = response['location']
+
+        content = "#Test Pingback - Source 1\n" + @base.to_s + target
+        response = post('message.rb', "title=#{CGI.escape(title)}&content=#{CGI.escape(content)}&format=#{CGI.escape('text/uri-list')}&parent=#{CGI.escape(parent)}&confirm", {"Cookie" => @@session})
+        @@session = response['set-cookie']
+        pingback1 = response['location']
+
+        content = "#Test Pingback - Source 2\n" + @base.to_s + target
+        response = post('message.rb', "title=#{CGI.escape(title)}&content=#{CGI.escape(content)}&format=#{CGI.escape('text/uri-list')}&parent=#{CGI.escape(parent)}&confirm", {"Cookie" => @@session})
+        @@session = response['set-cookie']
+        pingback2 = response['location']
+
+        # now check if both pingbacks are there
+        root = parse(get(target.to_s).body)
+        assert_equal [@base.to_s + pingback1, @base.to_s + pingback2],
+            elements(root, %{/html/body/div[@id="main"]//li/div/div[@class="content"]/p/a}, 'href')
+    end
 end
