@@ -20,10 +20,11 @@ class TestSession
         @login = 'test'
         @full_name = 'Test'
         @options = {}
+        @base = 'http://localhost' + config['site']['base'] + '/'
         bindtextdomain('samizdat', nil, 'C', 'utf-8')
     end
 
-    attr_accessor :id, :login, :full_name, :options
+    attr_accessor :id, :login, :full_name, :options, :base
 end
 
 class TC_Template < Test::Unit::TestCase
@@ -98,27 +99,23 @@ class TC_Template < Test::Unit::TestCase
         assert_equal ['file'], elements(root, 'div/input', 'type')
     end
 
-    def test_member_box
-        root = parse(@t.member_box)
-        assert_equal [@s.id.to_s, 'member.rb', 'logout.rb'],
-            elements(root, '//a', 'href')
-    end
-
-    def test_login_form
+    def test_member_line
         @s.id = nil
         @t = Template.new(@s)
-        root = parse(@t.member_box)
-        assert_equal ['Login'], elements(root, 'div[@class="box-title"]')
-        assert_equal ['login.rb'],
-            elements(root, 'div[@class="box-content"]/form', 'action')
-        assert_equal ['member.rb'],
-            elements(root, 'div[@class="box-content"]/p/a', 'href')
+        root = parse('<p>'+@t.member_line+'<p>')
+        assert_equal ['login.rb'], elements(root, '//a', 'href')
+    end
+
+    def test_member_line_registered
+        root = parse('<p>'+@t.member_line+'<p>')
+        assert_equal [@s.id.to_s, 'member.rb', 'logout.rb'],
+            elements(root, '//a', 'href')
     end
 
     def test_message_buttons
         root = parse(@t.message(:mode => :full,
             :full_name => 'Test', :title => 'Title'))
-        assert_equal ['title', 'parent', 'reply'],
+        assert_equal ['title', 'parent', 'reply', 'edit'],
             elements(root, 'div[@class="foot"]/form/div/input', 'name')
     end
 
@@ -171,20 +168,16 @@ class TC_Template < Test::Unit::TestCase
     end
 
     def test_simple_page
-        root = parse(@t.page('Test Title', 'Some text.', 'Some sidebar.'))
-        assert_equal [@s.login, 'Test Title'],
+        root = parse(@t.page('Test Title', 'Some text.'))
+        assert_equal ['Test Title'],
             elements(root, '//div[@class="box-title"]')
-        assert_equal 'Some sidebar.',
-            text(root, '/html/body/div[@id="sidebar"]')
     end
 
     def test_multi_page
         root = parse( @t.page('Title',
-            [ ['Title 1', 'Body 1.'], ['Title 2', 'Body 2.']],
-            'Some sidebar.'))
-        assert_equal ['Samizdat: Title'], elements(root, '/html/head/title')
-        assert_equal 'Some sidebar.',
-            text(root, '/html/body/div[@id="sidebar"]')
+            [ ['Title 1', 'Body 1.'], ['Title 2', 'Body 2.']]))
+        assert_equal [config['site']['name'] + ': Title'],
+            elements(root, '/html/head/title')
         main = root.elements['/html/body/div[@id="main"]']
         assert_equal ['Title 1', 'Title 2'],
             elements(main, 'div/div[@class="box-title"]')
